@@ -1,6 +1,6 @@
 
 package MLDBM::Sync;
-$VERSION = .03;
+$VERSION = .05;
 
 use MLDBM;
 use MLDBM::Sync::SDBM_File;
@@ -12,21 +12,22 @@ use Carp qw(confess);
 no strict qw(refs);
 use vars qw($AUTOLOAD @EXT $LOCK_SH $LOCK_EX $LOCK_UN);
 
-@EXT = ('.pag', '.dir', '');
 $LOCK_SH = LOCK_SH;
-$LOCK_EX = LOCK_EX;
 $LOCK_UN = LOCK_UN;
+$LOCK_EX = LOCK_EX;
+
+@EXT = ('.pag', '.dir', '');
 
 sub TIEHASH {
     my($class, $file, @args) = @_;
 
-    my $fh = "$file.lock";
-    open($fh, ">>$fh") || die("can't open file $fh: $!");
+    my $fh = $file.".lock";
 
     my $self = bless {
 		      'file' => $file,
 		      'args' => [ $file, @args ],
-		      'lock' => $fh,
+		      'lock_fh' => $fh,
+		      'lock_file' => $fh,
 		      'lock_num' => 0,
 		      'md5_keys' => 0,
 		      'pid' => $$,
@@ -40,12 +41,10 @@ sub TIEHASH {
 
 sub DESTROY { 
     my $self = shift;
-
-    $self->{dbm} = undef; # close / flush before unlock
-    if (($self->{'lock'})) {
-	close($self->{'lock'})
+    if($self->{lock_num}) {
+	$self->{lock_num} = 1;
+	$self->UnLock;
     }
-
 }
 
 sub AUTOLOAD {
@@ -157,31 +156,21 @@ sub SyncKeysChecksum {
 }
 
 *read_lock = *ReadLock;
-sub ReadLock { shift->Lock($LOCK_SH); }
+sub ReadLock { shift->Lock(1); }
 
 *lock = *SyncLock = *Lock;
 sub Lock {
-    my($self, $type) = @_;
-    $type ||= $LOCK_EX;
-
-    # FORK PROOF... we reinit the lock after a fork automatically this 
-    # way since a shared file handle will not produce correct results
-    # in a load test with forks involved otherwise.
-    if ($self->{pid} ne $$) {
-	my $fh = $self->{'lock'};
-	close $fh;
-	open($fh, ">>$fh") || die("can't open file $fh: $!");
-	$self->{pid} = $$;
-	$self->{lock_num} = 0;
-	$self->{lock_type} = undef;
-    }
+    my($self, $read_lock) = @_;
 
     if($self->{lock_num}++ == 0) {
-	flock($self->{'lock'}, $type) || die("can't write lock $self->{'lock'}: $!");
-	$self->{lock_type} = $type;
+	my $file = $self->{lock_file};
+	open($self->{lock_fh}, "+>$file") || die("can't open file $file: $!");
+	flock($self->{lock_fh}, ($read_lock ? $LOCK_SH : $LOCK_EX))
+	    || die("can't ". ($read_lock ? "read" : "write") ." lock $file: $!");
+	$self->{read_lock} = $read_lock;
 	$self->SyncTie;
     } else {
-	if (($self->{lock_type} eq $LOCK_SH) and ($type eq $LOCK_EX)) {
+	if ($self->{read_lock} and ! $read_lock) {
 	    $self->{lock_num}--; # roll back lock count
 	    # confess here to help developer track this down
 	    confess("can't upgrade lock type from LOCK_SH to LOCK_EX! ".
@@ -197,7 +186,9 @@ sub UnLock {
 
     if($self->{lock_num}-- == 1) {
 	undef $self->{dbm};
-	flock($self->{'lock'}, $LOCK_UN) || die("can't unlock $self->{'lock'}: $!");
+	flock($self->{'lock_fh'}, $LOCK_UN) || die("can't unlock $self->{'lock_file'}: $!");
+	close($self->{'lock_fh'}) || die("can't close $self->{'lock_file'}");
+	$self->{read_lock} = undef;
     } else {
 	1;
     }
